using System;
using System.Collections.Generic;
using System.Windows.Input;

namespace TestProject;

/// <summary>
/// 븮 
/// </summary>
/// <typeparam name="TParameter">Ű  Ÿ</typeparam>
public class DelegateCommand<TParameter> : ICommand
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Event
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region     ̺Ʈ - CanExecuteChanged

    /// <summary>
    ///     ̺Ʈ
    /// </summary>
    public event EventHandler CanExecuteChanged
    {
        add
        {
            CommandManager.AddWeakReferenceHandler(ref this.weakReferenceList, value);
        }
        remove
        {
            CommandManager.RemoveWeakReferenceHandler(this.weakReferenceList, value);
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    ///  ׼
    /// </summary>
    private readonly Action<TParameter> executeAction;

    /// <summary>
    ///    Լ
    /// </summary>
    private readonly Func<TParameter, bool> canExecuteFunction;

    /// <summary>
    ///   Ʈ
    /// </summary>
    private List<WeakReference> weakReferenceList;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - DelegateCommand(executeAction, canExecuteFunction)

    /// <summary>
    /// 
    /// </summary>
    /// <param name="executeAction"> ׼</param>
    /// <param name="canExecuteFunction">   Լ</param>
    public DelegateCommand(Action<TParameter> executeAction, Func<TParameter, bool> canExecuteFunction)
    {
        if(executeAction == null)
        {
            throw new ArgumentNullException("executeAction");
        }

        this.executeAction      = executeAction;
        this.canExecuteFunction = canExecuteFunction;
    }

    #endregion
    #region  - DelegateCommand(executeAction)

    /// <summary>
    /// 
    /// </summary>
    /// <param name="executeAction"> ׼</param>
    public DelegateCommand(Action<TParameter> executeAction) : this(executeAction, null)
    {
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region    ϱ - CanExecute(parameter)

    /// <summary>
    ///    ϱ
    /// </summary>
    /// <param name="parameter">Ű </param>
    /// <returns>  </returns>
    public bool CanExecute(TParameter parameter)
    {
        if(this.canExecuteFunction != null)
        {
            return this.canExecuteFunction(parameter);
        }

        return true;
    }

    #endregion
    #region ϱ - Execute(parameter)

    /// <summary>
    /// ϱ
    /// </summary>
    /// <param name="parameter">Ű </param>
    public void Execute(TParameter parameter)
    {
        if(this.executeAction != null)
        {
            this.executeAction(parameter);
        }
    }

    #endregion

    #region    ϱ - ICommand.CanExecute(parameter)

    /// <summary>
    ///    ϱ
    /// </summary>
    /// <param name="parameter">Ű </param>
    /// <returns>  </returns>
    bool ICommand.CanExecute(object parameter)
    {
        if(parameter == null && typeof(TParameter).IsValueType)
        {
            return this.canExecuteFunction == null;
        }

        return CanExecute((TParameter)parameter);
    }

    #endregion
    #region ϱ - ICommand.Execute(parameter)

    /// <summary>
    /// ϱ
    /// </summary>
    /// <param name="parameter">Ű </param>
    void ICommand.Execute(object parameter)
    {
        Execute((TParameter)parameter);
    }

    #endregion

    #region     ̺Ʈ ߻Ű - RaiseCanExecuteChangedEvent()

    /// <summary>
    ///     ̺Ʈ ߻Ű
    /// </summary>
    public void RaiseCanExecuteChangedEvent()
    {
        FireCanExecuteChangedEvent();
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Protected

    #region     ̺Ʈ ߻Ű - FireCanExecuteChangedEvent()

    /// <summary>
    ///     ̺Ʈ ߻Ű
    /// </summary>
    protected virtual void FireCanExecuteChangedEvent()
    {
        CommandManager.CallWeakReferenceHandlers(this.weakReferenceList);
    }

    #endregion
}